#ifndef __KYLIN_HA_ACTIVATION_H__ 
#define __KYLIN_HA_ACTIVATION_H__

#define SERVER_ADDR "wx.kylinos.cn"                             //服务器域名
#define SERVER_PORT 80                                          //服务器端口
#define BUF_SIZE 1024                                           //缓冲区大小
#define HA_VERSION "Kylin HA Cluster Software V10(Cactus)"      //HA版本
#define QR_PNG 1                                                //以PNG方式生成二维码
#define QR_ANSI 2                                               //以ANSI形式生成二维码
#define DAY_SEC (60*60*24)                                      //一天的秒数
#define ADDED_TRIAL_NUM "kylin_ha_cluster_software_v10"         //试用延期后将此字符串保存到.ky_ha_trial
#define QR_PNG_PATH "/tmp/kylin_ha_qr.png"                      //保存生成二维码图片路径
#define HA_INST_FILE "/etc/corosync/.ky_ha_inst"                //记录加密后的安装时间
#define KYLIN_INFORMATION_FILE "/etc/corosync/.kyinfo"          //授权文件
#define KYLIN_INFORMATION_FILE_BAK "/tmp/.kyinfo_old"           //旧授权文件备份
#define KYLIN_ACTIVATION_FILE "/etc/corosync/.kyactivation"     //已激活信息记录文件
#define KYLIN_LICENSE_FILE "/etc/corosync/LICENSE"              //授权文件
#define KYLIN_LICENSE_FILE_BAK "/tmp/LICENSE_old"               //旧授权文件备份
#define HA_MD5_FILE "/etc/corosync/.ky_ha_lic"                  //记录LICENSE文件加密后的md5字符串
#define HA_TRIAL_FILE "/etc/corosync/.ky_ha_trial"              //记录加密后试用期截至日期
#define HA_KYINFO_KEY_FILE "/etc/corosync/.ky_ha_key"           //保存首次安装kyinfo中加密后的key字段
#define HA_WEB_PLACE_LIC "/tmp/LICENSE"                         //通过web上传场地授权文件路径
#define HA_WEB_PLACE_KYINFO "/tmp/.kyinfo"                      //通过web上传场地授权文件路径
#define HA_CUR_TIME "/etc/corosync/.ky_ha_tm"                   //保存调用接口时当前时间

enum
{
	ON_LINE = 1,    //网络状态处于在线
	OFF_LINE        //网络状态处于离线
};



//ha授权相关状态检查
struct _ha_check_res
{
	bool lic_flag;                                          //LICENSE检查
	bool kyinfo_flag;                                       //kyinfo检查
	bool inst_time_flag;                                    //安装时间检查
	bool trial_flag;                                        //试用期检查
	bool lic_md5_flag;                                      //LICENSE md5检查
	bool added_trial_flag;                                  //是否延期过试用期
	bool trial_lic_flag;                                    //延期授权文件检查
};

//json数据
struct _ha_json_stru
{
	char serial[BUF_SIZE];                                  //服务序列号
	char user[BUF_SIZE];                                    //用户信息
	char key[BUF_SIZE];                                     //激活码
	int errcode;                                            //错误码
	char errmsg[BUF_SIZE];                                  //错误信息
};

//激活方式
enum ha_activate_way
{
	QR = 1,                                                 //二维码激活
	AREA,                                                   //场地激活
	UNKNOWN                                                 //未知方式
};

//处于的激活状态
enum ha_activate_status
{
	IN_TRIAL = 1,                                           //试用期
	OUT_TRIAL_NOT_ACTIVATED,                                //试用过期，未激活
	IS_ACTIVATED                                            //已激活
};

//LICENSE中各个字段
struct _ha_license
{
	char to_str[BUF_SIZE];                                  //用户信息
	char serial_str[BUF_SIZE];                              //生产批次号
	char term_str[BUF_SIZE];                                //试用期字段（未用，license中计划删除此字段）
	char class_str[BUF_SIZE];                               //版本形态 如：HA
	char version_str[BUF_SIZE];                             //版本
	char platform_str[BUF_SIZE];                            //体系架构 
	char method_str[BUF_SIZE];                              //授权方式 二维码授权文件无此字段，场地授权特有此字段，且值为place
	char old_key_str[BUF_SIZE];                             //服务延期授权文件特有字段 记录旧服务序列号
};

//kyinfo中各个字段
struct _ha_kyinfo
{
	char dist_name[BUF_SIZE];
	char dist_milestone[BUF_SIZE];
	char dist_arch[BUF_SIZE];
	char dist_beta[BUF_SIZE];
	char dist_time[BUF_SIZE];
	char dist_dist_id[BUF_SIZE];
	char servicekey_key[BUF_SIZE];
	char os_to[BUF_SIZE];
	char os_term[BUF_SIZE];
	char installer_kylin_type[BUF_SIZE];
};

//激活信息
struct _ha_activation_info
{
	char version[BUF_SIZE];                                 //版本
	char serial[BUF_SIZE];                                  //服务序列号
	int activate_status;                                    //激活状态
	int remain_trial;                                       //剩余试用期
	int activate_way;                                       //激活方式
	char customer[BUF_SIZE];                                //用户信息
	int added_trial;                                        //是否已延期过
};

//激活信息结构体初始化
struct _ha_activation_info * ha_activation_info_init(void);

//获取剩余试用期时间
//返回 失败:-1 已过试用期（未激活）:-99999 试用期中:剩余试用期时间
int get_remain_trial(void);

//生成延期后的试用期时间
//返回 失败:-1 成功:0
int gen_last_trial_time(void);

//获取与服务器联网状态
//返回 已联网:ON_LINE 未联网:OFF_LINE 失败:-1
int get_net_status(void);

//将激活码字符串格式化为AAAA-BBBB-CCCC-DDDD-EEEE
//输入参数：
//input_str: 输入的激活码
//output_str: 返回已格式化激活码
//output_len: 已格式化字符串长度
//返回 成功
int format_activation_code(const char *input_str, char *output_str, int output_len);

//解析从服务器返回的激活json数据
//输入参数：
//json_str: 服务器传回的json字符串
//json_stru: 保存json数据
//返回 成功:0 失败:-1
int parse_json(const char *json_str, struct _ha_json_stru **json_stru);

//curl接收处理函数
//返回 接收数据长度
size_t write_function(void *ptr, size_t size, size_t nmemb, void *stream);

//二维码离线激活
//返回 成功0 失败-1
int off_line_activate(void);

//二维码在线激活
//返回 成功0 失败-1
int on_line_activate(void);

//激活
//输入参数：
//serial_num: 服务序列号
//activation_code: 激活码（带'-'）
//返回 成功0 失败-1
int start_ha_activate(const char *serial_num, const char *activation_code);

//获取HA版本
//输入参数：
//ha_version: 返回HA版本字符串
//ha_version_len: HA版本字符串长度
void get_ha_version(char *ha_version, int ha_version_len);

//获取HA序列号
//输入参数：
//ha_serial: 返回HA序列号
//ha_serial: 输入字符串的长度
//返回 成功0 失败-1
int get_ha_serial_num(char * ha_serial, int ha_serial_len);

//获取ha激活状态:试用期 使用过期 已激活 试用期时间
//输入参数：
//status: 返回激活状态
//remain_trial: 返回剩余试用期时间
//返回 成功0 失败-1
int get_ha_activate_status_trial(int *status, int *remain_trial);

//获取HA激活方式
//输入参数：
//activate_status: 传入激活状态
//返回 场地授权：AREA 二维码授权：QR 未知状态：UNKNOWN
int get_ha_activate_way(int activate_status);

//获取HA激活信息
//输入参数：
//ha_acitvation_info：保存激活信息
//返回 成功0 失败-1
int get_ha_activate_info(struct _ha_activation_info *ha_activation_info);

//创建二维码
//输入参数：
//str：需要转换为二维码的字符串
//type：二维码类型 QR_PNG QR_ANSI
//返回成功0 失败-1
int create_QR(const char *str, int type);

//解密
//输入参数：
//en_str: 加密字符串
//par_str: 解密后字符串
//par_str_len: 解密字符串长度
//返回 成功0 失败-1
int ha_decrypt(const void *en_str, char*par_str, int par_str_len);

//加密
//输入参数：
//src_str: 源字符串
//en_str: 加密后字符串
//en_str_len: 加密字符串长度
//返回 成功加密后字符串长度 失败-1
int ha_encrypt(const char *src_str, char *en_str, int en_str_len);

//试用期文件检查
//输入参数：
//res: 返回保存的标志信息
//返回 成功0 失败-1
int add_trial_file_check(struct _ha_check_res *res);

//生成初次安装的试用期时间
//返回 成功0 失败-1
int gen_first_inst_trial_time(void);

//将file_path生成md5
//输入参数：
//file_path: 需要生成md5的文件路径
//md5_str: 生成的md5
//md5_len: md5字符串长度
//返回 成功0 失败-1
int gen_md5(const char *file_path, unsigned char *md5_str, int md5_len);

//将file_path中加密字符串解密
//输入参数：
//file_path: 保存加密字符串文件
//返回 成功：解密后字符串 失败：NULL
char * read_decrypt_file_to_str(const char *file_path);

//将字符串加密并写入到file_path中
//输入参数：
//file_path: 加密字符串保存路径
//src_str: 需要加密的明文字符串
//返回 成功0 失败-1
int write_encrypt_str_to_file(const char *file_path, const char *src_str);

//试用期延期
//返回 成功0 失败-1
int add_trial(void);

//检查是否已经延期过
//输入参数：
//added_status: 返回是否延期状态 已延期过1 未延期过0
//返回 成功0 失败-1
int check_added_trial(int *added_status);

//web端在线激活 返回1已激活 0未激活 -1错误
//返回
int web_online_activation(void);

//web端离线激活 返回1已激活 0未激活 -1错误
//输入参数：
//返回
int web_offline_activation(const char *serial, const char *activation_code);

//web端场地授权激活失败环境重置
//返回
int web_place_restore(void);

//web端场地授权函数 
//返回 已激活1  未激活0  错误-1
int web_place_activation(void);

//复制文件
//输入参数：
//src_path: 源路径
//dist_path: 目的路径
//返回 成功0 失败-1
int copy_file(const char *src_path, const char *dist_path);

//场地授权获取用户信息
//返回 成功：用户信息字符串  失败：NULL
char * place_get_custom(void);

//从服务器获取激活数据
//输入参数：
//json_stru: 保存json信息
//返回 成功0 失败-1
int load_json_from_url(struct _ha_json_stru *json_stru);

//.kyinfo结构体初始化
struct _ha_kyinfo * ha_kyinfo_init(void);

//LICENSE结构体初始化
struct _ha_license * ha_license_init(void);

//.kyinfo结构体释放内存
int ha_kyinfo_destroy(struct _ha_kyinfo **kyinfo);

//LICENSE结构体释放内存
int ha_license_destroy(struct _ha_license **lic);

//写入数据到LICENSE结构体，如果对应字段无值，赋值为""
//输入参数：
//lic: 保存LICENSE内容
//to: 输入to字段
//serial: 输入serial字段
//term: 输入term字段
//class: 输入class字段
//version: 输入version字段
//platform: 输入platform字段
//method: 输入method字段
//old_key: 输入old_key字段
//返回 成功0 失败-1
int write_to_lic_stru(struct _ha_license **lic, const char *to, const char *serial, const char *term, const char *class, const char *version, const char *platform, const char *method, const char *old_key);

//写入数据到LICENSE结构体，如果对应字段无值，赋值为""
//输入参数：
//info: 保存kyinfo内容
//后续参数均为kyinfo文件中字段
int write_to_kyinfo_stru(struct _ha_kyinfo **info, const char *name, const char *milestone, const char *arch, const char *beta, const char *time, const char *dist_id, const char *key,      const char *to, const char *term, const char *kylin_type);

//读取LICENSE
//输入参数：
//lic: 保存LICENSE内容
//返回 成功0 失败-1
int load_license(struct _ha_license **lic);

//读取.kyinfo
//输入参数：
//kyinfo: 保存.kyinfo内容
//返回 成功0 失败-1
int load_kyinfo(struct _ha_kyinfo **kyinfo);

//检查当前时间，并更新到文件
//返回 成功0 失败-1
int check_and_update_time(void);

//LICENSE与.kyinfo内容有效性检查
//返回成功0 失败-1
int LICENSE_kyinfo_check(void);

#endif
